<?php

declare(strict_types=1);

namespace Plugin\cin_vivawallet_5;

require_once __DIR__.'/vendor/autoload.php';

use Cin\Vivawallet5\Licence;
use Cin\Vivawallet5\LicenceWidget;
use Cin\Vivawallet5\Plugin;
use JTL\Backend\NotificationEntry;
use JTL\Events\Dispatcher;
use JTL\Plugin\Bootstrapper;
use JTL\Shop;

final class Bootstrap extends Bootstrapper
{
  public const FLAG_DELETABLE = 1;

  private $P;

  public function boot(Dispatcher $dispatcher): void
  {
    parent::boot($dispatcher);
    // listen for admin calls
    if (0 === Shop::getPageType() && str_starts_with($_SERVER['REQUEST_URI'], URL_SHOP.'/admin/')) {
      $L = new Licence();
      if ( ! $L->canRun()) {
        $this->licenceNotification();
      }
    }

    Shop::Container()->getGetText()->setLanguage()->loadTranslations($this->P()->getJTlPlugin()->getPaths()->getBasePath(), 'custom');
  }

  public function installed(): void
  {
    LicenceWidget::initializeWidget();
  }

  public function enabled(): void
  {
    LicenceWidget::initializeWidget();
  }

  public function disabled(): void
  {
  }

  public function uninstalled($deleteData = true): void
  {
    LicenceWidget::checkRemoveWidget();
    $this->unsetSession();
  }

  public function updated($oldVersion, $newVersion): void
  {
    LicenceWidget::initializeWidget();
    $this->unsetSession();
  }

  private function licenceNotification(): void
  {
    $this->addNotify(
      NotificationEntry::TYPE_DANGER,
      $this->getPlugin()->getMeta()->getName(),
      '<a href="https://www.com-ins-netz.de/Support-Hilfecenter">Es gibt Probleme mit der Lizenz. Wenden Sie sich an com-ins-netz.de</a>'
    );
  }

  private function P(): Plugin
  {
    return $this->P ?? ($this->P = new Plugin());
  }

  private function unsetSession(): void
  {
    unset(
      $_SESSION["{$this->getPlugin()->getPluginID()}_versionResponse"],
      $_SESSION["{$this->getPlugin()->getPluginID()}_productHomepage"]
    );
  }
}
